package testzilla;
#---------------------------------------------------------------
# Project         : Mandrake Linux
# Module          : testzilla/clients
# File            : testzilla.pm
# Version         : $Id: testzilla.pm,v 1.12 2004/01/29 19:40:09 flepied Exp $
# Author          : Frederic Lepied
# Created On      : Wed Jan  7 20:48:18 2004
# Purpose         : common functions
#---------------------------------------------------------------

use SOAP::Lite;
use POSIX;
use Term::ReadKey;

use lib qw(/usr/lib/libDrakX);
use detect_devices;
use standalone;

use MDK::Common;
use printer::detect;

my $user = $ARGV[0];
my $pass;
my @components;

sub int16 {
    my ($n) = @_;
    return sprintf("%04x", $n);
}

sub SOAP::Transport::HTTP::Client::get_basic_credentials { 
    return $user => $pass;
}

# detection routines

sub detect {
    my ($type) = @_;
    
    require harddrake::data;
    foreach (@harddrake::data::tree) {
        my ($Ident, $item, undef, $configurator, $detector, $do_it) = @$_;

	next if ($type && $type ne $Ident);
	
        # No detector ? (should never happen but who knows ?)
        ref($detector) eq 'CODE' or next;
	
        my @devices;

	# Detect only local devices
	if ($Ident eq 'PRINTER') {
	    @devices = printer::detect::local_detect();
	} else {
	    @devices = &$detector;
	}
	
	return @devices if ($type);

        @devices or next;

        # FIXME: fill me
	use Data::Dumper;
	print 'Probing ', $Ident, " class\n";
	print Dumper(\@devices);
    }
}

sub upload_video_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('VIDEO')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	print "Uploading video card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	$r = $hwdb->add_video_card($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
	print "done [", $r->result, "]\n";
	push(@components, $r->result);
    }
}

sub upload_network_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('ETHERNET')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	if ($card->{bus} eq 'PCI') {
	    print "Uploading PCI network card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	    $r = $hwdb->add_pci_network_card($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
	} elsif ($card->{bus} eq 'USB') {
	    print "Uploading USB network card $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_network_card($card->{description}, int16($card->{vendor}), int16($card->{id}));
	} else {
	    print "unknown bus (ethernet): $card->{bus} $card->{description} ($card->{vendor}:$card->{id})\n";
	}
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_sound_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('AUDIO')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	if ($card->{bus} eq 'PCI') {
	    print "Uploading PCI audio card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	    $r = $hwdb->add_pci_sound_card($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
	} elsif ($card->{bus} eq 'USB') {
	    print "Uploading USB audio card $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_sound_card($card->{description}, int16($card->{vendor}), int16($card->{id}));
	} else {
	    print "unknown bus (audio): $card->{bus} $card->{description} ($card->{vendor}:$card->{id})\n";
	}
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_scanner {
    my ($hwdb) = @_;

    foreach my $info (detect('SCANNER')) {
	my $r;
	my $card = $info->{val};
	
	$card->{DESCRIPTION} =~ s/.*\|//;
	if ($info->{port} =~ '/usb/') {
	    $card->{vendor} =~ s/0x//;
	    $card->{id}  =~ s/0x//;
	    print "Uploading USB scanner $card->{DESCRIPTION} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_scanner($card->{DESCRIPTION}, $card->{vendor}, $card->{id});
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_printer {
    my ($hwdb) = @_;

    foreach my $info (detect('PRINTER')) {
	my $r;
	my $card = $info->{val};

	$card->{MODEL} =~ s/.*\|//;
	if ($info->{port} =~ /usb/) {
	    $card->{vendor} =~ s/0x//;
	    $card->{id}  =~ s/0x//;
	    print "Uploading USB printer $card->{MODEL} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_printer($card->{MODEL}, $card->{vendor}, $card->{id});
	} elsif ($info->{port} =~ m@/dev/lp@) {
	    print "Uploading parallel printer $card->{MODEL}...";
	    $r = $hwdb->add_parallel_printer($card->{MODEL}, $card->{MANUFACTURER});
	} else {
	    print "unknown printer port: $info->{port}\n";
	}
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_dmi {
    my ($hwdb) = @_;
    my $info;
    my $r;
    
    foreach (`dmidecode`) {
	chomp($_);
	
	# BIOS info
	if (/^\tDMI type 0,/ ... /^Handle/) {
	    $info->{bios} = $2 if (/.*(vendor|manufacturer):\s*(\S.+?)\s*$/i);
	    $info->{biosver} = $1 if (/.*Version:\s*(\S.+?)\s*$/i);
	    $info->{date} = $1 if (/.*Date:\s*(\S.+?)\s*$/i);
	    if (/^Handle/ && $info->{bios} && $info->{biosver}) {
		print "Uploading BIOS $info->{bios} ($info->{biosver}:$info->{date})...";
		$r = $hwdb->add_bios($info->{bios}, $info->{biosver}, $info->{date});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{bios} = 0;
	    }
	}
	
	# system info
	if (/^\tDMI type 1,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if (/.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i);
	    $info->{version} = $1 if (/.*Version:\s*(\S.+?)\s*$/i);
	    $info->{product} = $1 if (/.*Product.*:\s*(\S.+?)\s*$/i);
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading system $info->{manufacturer}:$info->{product}:$info->{version}...";
		if ($info->{version}) {
		    $r = $hwdb->add_system_info($info->{manufacturer}, $info->{version}, $info->{product});
		} else {
		    $r = $hwdb->add_system_info($info->{manufacturer}, $info->{product}, $info->{version});
		}
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
		$info->{version} = 0;
	    }
	}

	# board info
	if (/^\tDMI type 2,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if (/.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i);
	    $info->{product} = $1 if (/.*Product.*:\s*(\S.+?)\s*$/i);
	    $info->{version} = $1 if (/.*Version:\s*(\S.+?)\s*$/i);
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading board $info->{manufacturer}:$info->{product}:$info->{version}...";
		$r = $hwdb->add_board($info->{manufacturer}, $info->{product}, $info->{version});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
	    }
	}

	# Chassis info
	if (/^\tDMI type 3,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if (/.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i);
	    $info->{type} = $1 if (/.*Type*:\s*(\S.+?)\s*$/i);
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading chassis $info->{manufacturer}:$info->{type}...";
		$r = $hwdb->add_chassis($info->{manufacturer}, $info->{type});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
	    }
	}
    }
}

sub upload_system {
    my ($hwdb) = @_;
    
    upload_dmi($hwdb);
    upload_video_card($hwdb);
    upload_network_card($hwdb);
    upload_sound_card($hwdb);
    upload_scanner($hwdb);
    upload_printer($hwdb);
}

# common init phase

sub init {
    my ($name) = @_;
    
    if ($#ARGV == -1) {
	print "usage: $name <bugzilla account>\n";
	exit 1;
    }

    my $hwdb= SOAP::Lite
	-> uri('http://qa.mandrakesoft.com/HW')
	-> proxy('http://qa.mandrakesoft.com/HW/hw.cgi')
	-> on_fault(sub { my($soap, $res) = @_;
			  die ref $res ? $res->faultstring : $soap->transport->status, "\n";
		      });

    POSIX::isatty(fileno(STDIN)) or die "not a tty\n";

    print "Bugzilla password for $user: ";
    ReadMode('noecho');
    $pass = ReadLine 0;
    chomp $pass;
    ReadMode('normal');
    print "\n";

    return $hwdb
}

1;

# testzilla.pm ends here
